Class: Pry::CommandSet

Inherits:
Object show all
Includes:
Enumerable, Helpers::BaseHelpers
Defined in:
lib/pry/command_set.rb

Overview

This class is used to create sets of commands. Commands can be imported from different sets, aliased, removed, etc.

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Helpers::BaseHelpers

colorize_code, #colorize_code, command_dependencies_met?, #command_dependencies_met?, create_command_stub, #create_command_stub, find_command, #gem_installed?, gem_installed?, #heading, heading, highlight, #highlight, jruby?, #jruby?, #lesspipe, lesspipe, #mri_18?, mri_18?, mri_19?, #mri_19?, not_a_real_file?, #not_a_real_file?, #page_size, page_size, #rbx?, rbx?, set_file_and_dir_locals, #set_file_and_dir_locals, silence_warnings, #silence_warnings, #simple_pager, simple_pager, stagger_output, #stagger_output, stub_proc, #stub_proc, use_ansi_codes?, #use_ansi_codes?, windows?, #windows?, windows_ansi?, #windows_ansi?

Constructor Details

#initialize(*imported_sets) { ... } ⇒ CommandSet

Returns a new instance of CommandSet.

Parameters:

  • imported_sets (Array<CommandSet>)

    Sets which will be imported automatically

Yields:

  • Optional block run to define commands



20
21
22
23
24
25
26
27
# File 'lib/pry/command_set.rb', line 20

def initialize(*imported_sets, &block)
  @commands      = {}
  @helper_module = Module.new

  import(*imported_sets)

  instance_eval(&block) if block
end

Instance Attribute Details

#commandsObject (readonly)

Returns the value of attribute commands.



14
15
16
# File 'lib/pry/command_set.rb', line 14

def commands
  @commands
end

#helper_moduleObject (readonly)

Returns the value of attribute helper_module.



15
16
17
# File 'lib/pry/command_set.rb', line 15

def helper_module
  @helper_module
end

Instance Method Details

#add_command(command) ⇒ Object

Add a given command object to this set.

Parameters:

  • command (Command)

    The subclass of Pry::Command you wish to add.



157
158
159
# File 'lib/pry/command_set.rb', line 157

def add_command(command)
  commands[command.match] = command
end

#after_command(search) { ... } ⇒ Object

Execute a block of code after a command is invoked. The block also gets access to parameters that will be passed to the command and is evaluated in the same context.

Examples:

Display text 'command complete' after invoking command

Pry.commands.after_command("whereami") do |n|
  output.puts "command complete!"
end

Parameters:

  • search (String, Regexp)

    The match or listing of the command.

Yields:

  • The block to be run after the command.



146
147
148
149
# File 'lib/pry/command_set.rb', line 146

def after_command(search, &block)
  cmd = find_command_by_match_or_listing(search)
  cmd.hooks[:after] << block
end

#alias_command(match, action, options = {}) ⇒ Object

Aliases a command

Examples:

Creating an alias for ls -M

Pry.config.commands.alias_command "lM", "ls -M"

Pass explicit description (overriding default).

Pry.config.commands.alias_command "lM", "ls -M", :desc => "cutiepie"

Parameters:

  • match (String, Regex)

    The match of the alias (can be a regex).

  • action (String)

    The action to be performed (typically another command).

  • options (Hash) (defaults to: {})

    The optional configuration parameters, accepts the same as the command method, but also allows the command description to be passed this way too as :desc



221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
# File 'lib/pry/command_set.rb', line 221

def alias_command(match, action,  options={})
  original_options = find_command(action).options.dup

  options = original_options.merge!({
                                      :desc => "Alias for `#{action}`",
                                      :listing => match
                                    }).merge!(options)

  # ensure default description is used if desc is nil
  desc = options.delete(:desc).to_s

  c = block_command match, desc, options do |*args|
    run action, *args
  end

  c.group "Aliases"

  c
end

#before_command(search) { ... } ⇒ Object

Execute a block of code before a command is invoked. The block also gets access to parameters that will be passed to the command and is evaluated in the same context.

Examples:

Display parameter before invoking command

Pry.commands.before_command("whereami") do |n|
  output.puts "parameter passed was #{n}"
end

Parameters:

  • search (String, Regexp)

    The match or listing of the command.

Yields:

  • The block to be run before the command.



132
133
134
135
# File 'lib/pry/command_set.rb', line 132

def before_command(search, &block)
  cmd = find_command_by_match_or_listing(search)
  cmd.hooks[:before].unshift block
end

#block_command(match, description = "No description.", options = {}) { ... } ⇒ Object Also known as: command

Defines a new Pry command.

Examples:

MyCommands = Pry::CommandSet.new do
  command "greet", "Greet somebody" do |name|
    puts "Good afternoon #{name.capitalize}!"
  end
end

# From pry:
# pry(main)> _pry_.commands = MyCommands
# pry(main)> greet john
# Good afternoon John!
# pry(main)> help greet
# Greet somebody

Regexp command

MyCommands = Pry::CommandSet.new do
  command /number-(\d+)/, "number-N regex command", :listing => "number" do |num, name|
    puts "hello #{name}, nice number: #{num}"
  end
end

# From pry:
# pry(main)> _pry_.commands = MyCommands
# pry(main)> number-10 john
# hello john, nice number: 10
# pry(main)> help number
# number-N regex command

Parameters:

  • match (String, Regexp)

    The start of invocations of this command.

  • description (String) (defaults to: "No description.")

    A description of the command.

  • options (Hash) (defaults to: {})

    The optional configuration parameters.

Options Hash (options):

  • :keep_retval (Boolean)

    Whether or not to use return value of the block for return of command or just to return nil (the default).

  • :requires_gem (Array<String>)

    Whether the command has any gem dependencies, if it does and dependencies not met then command is disabled and a stub proc giving instructions to install command is provided.

  • :interpolate (Boolean)

    Whether string #{} based interpolation is applied to the command arguments before executing the command. Defaults to true.

  • :listing (String)

    The listing name of the command. That is the name by which the command is looked up by help and by show-command. Necessary for commands with regex matches.

  • :use_prefix (Boolean)

    Whether the command uses Pry.config.command_prefix prefix (if one is defined). Defaults to true.

  • :shellwords (Boolean)

    Whether the command's arguments should be split using Shellwords instead of just split on spaces. Defaults to true.

Yields:

  • The action to perform. The parameters in the block determines the parameters the command will receive. All parameters passed into the block will be strings. Successive command parameters are separated by whitespace at the Pry prompt.



82
83
84
85
86
87
# File 'lib/pry/command_set.rb', line 82

def block_command(match, description="No description.", options={}, &block)
  description, options = ["No description.", description] if description.is_a?(Hash)
  options = default_options(match).merge!(options)

  commands[match] = Pry::BlockCommand.subclass(match, description, options, helper_module, &block)
end

#create_command(match, description = "No description.", options = {}) { ... } ⇒ Object

Defines a new Pry command class.

Examples:

Pry::Commands.create_command "echo", "echo's the input", :shellwords => false do
  def options(opt)
    opt.banner "Usage: echo [-u | -d] <string to echo>"
    opt.on :u, :upcase, "ensure the output is all upper-case"
    opt.on :d, :downcase, "ensure the output is all lower-case"
  end

  def process
    raise Pry::CommandError, "-u and -d makes no sense" if opts.present?(:u) && opts.present?(:d)
    result = args.join(" ")
    result.downcase! if opts.present?(:downcase)
    result.upcase! if opts.present?(:upcase)
    output.puts result
  end
end

Parameters:

  • match (String, Regexp)

    The start of invocations of this command.

  • description (String) (defaults to: "No description.")

    A description of the command.

  • options (Hash) (defaults to: {})

    The optional configuration parameters, see #command

Yields:

  • The class body's definition.



114
115
116
117
118
119
120
121
# File 'lib/pry/command_set.rb', line 114

def create_command(match, description="No description.", options={}, &block)
  description, options = ["No description.", description] if description.is_a?(Hash)
  options = default_options(match).merge!(options)

  commands[match] = Pry::ClassCommand.subclass(match, description, options, helper_module, &block)
  commands[match].class_eval(&block)
  commands[match]
end

#default_options(match) ⇒ Object (private)



353
354
355
356
357
358
359
360
361
362
363
364
# File 'lib/pry/command_set.rb', line 353

def default_options(match)
  {
    :requires_gem => [],
    :keep_retval => false,
    :argument_required => false,
    :interpolate => true,
    :shellwords => true,
    :listing => (String === match ? match : match.inspect),
    :use_prefix => true,
    :takes_block => false
  }
end

#delete(*searches) ⇒ Object

Removes some commands from the set

Parameters:

  • searches (Array<String>)

    the matches or listings of the commands to remove



163
164
165
166
167
168
# File 'lib/pry/command_set.rb', line 163

def delete(*searches)
  searches.each do |search|
    cmd = find_command_by_match_or_listing(search)
    commands.delete cmd.match
  end
end

#desc(search, description = nil) ⇒ Object

Sets or gets the description for a command (replacing the old description). Returns current description if no description parameter provided.

Examples:

Setting

MyCommands = Pry::CommandSet.new do
  desc "help", "help description"
end

Getting

Pry.config.commands.desc "amend-line"

Parameters:

  • search (String, Regexp)

    The command match.

  • description (String?) (defaults to: nil)

    (nil) The command description.



276
277
278
279
280
281
# File 'lib/pry/command_set.rb', line 276

def desc(search, description=nil)
  cmd = find_command_by_match_or_listing(search)
  return cmd.description if !description

  cmd.description = description
end

#each(&block) ⇒ Object



151
152
153
# File 'lib/pry/command_set.rb', line 151

def each &block
  @commands.each(&block)
end

#find_command(val) ⇒ Pry::Command? Also known as: []

Find a command that matches the given line

Parameters:

  • val (String)

    The line that might be a command invocation

Returns:



308
309
310
# File 'lib/pry/command_set.rb', line 308

def find_command(val)
  commands.values.select{ |c| c.matches?(val) }.sort_by{ |c| c.match_score(val) }.last
end

#find_command_for_help(search) ⇒ Pry::Command?

Find the command that the user might be trying to refer to.

Parameters:

  • search (String)

    The user's search.

Returns:



316
317
318
319
320
321
322
# File 'lib/pry/command_set.rb', line 316

def find_command_for_help(search)
  find_command(search) || (begin
    find_command_by_match_or_listing(search)
  rescue ArgumentError
    nil
  end)
end

#helpers { ... } ⇒ Object

Defines helpers methods for this command sets. Those helpers are only defined in this command set.

Examples:

helpers do
  def hello
    puts "Hello!"
  end

  include OtherModule
end

Yields:

  • A block defining helper methods



295
296
297
# File 'lib/pry/command_set.rb', line 295

def helpers(&block)
  helper_module.class_eval(&block)
end

#import(*sets) ⇒ Pry::CommandSet

Imports all the commands from one or more sets.

Parameters:

  • sets (Array<CommandSet>)

    Command sets, all of the commands of which will be imported.

Returns:



174
175
176
177
178
179
180
# File 'lib/pry/command_set.rb', line 174

def import(*sets)
  sets.each do |set|
    commands.merge! set.commands
    helper_module.send :include, set.helper_module
  end
  self
end

#import_from(set, *matches) ⇒ Pry::CommandSet

Imports some commands from a set

Parameters:

  • set (CommandSet)

    Set to import commands from

  • matches (Array<String>)

    Commands to import

Returns:



186
187
188
189
190
191
192
193
# File 'lib/pry/command_set.rb', line 186

def import_from(set, *matches)
  helper_module.send :include, set.helper_module
  matches.each do |match|
    cmd = set.find_command_by_match_or_listing(match)
    commands[cmd.match] = cmd
  end
  self
end

#list_commandsArray

Returns The list of commands provided by the command set.

Returns:

  • (Array)

    The list of commands provided by the command set.



301
302
303
# File 'lib/pry/command_set.rb', line 301

def list_commands
  commands.keys
end

#process_line(val, context = {}) ⇒ CommandSet::Result

Process the given line to see whether it needs executing as a command.

Parameters:

  • val (String)

    The line to execute

  • context (Hash) (defaults to: {})

    The context to execute the commands with

Returns:

  • (CommandSet::Result)


335
336
337
338
339
340
341
342
343
# File 'lib/pry/command_set.rb', line 335

def process_line(val, context={})
  if command = find_command(val)
    context = context.merge(:command_set => self)
    retval = command.new(context).process_line(val)
    Result.new(true, retval)
  else
    Result.new(false)
  end
end

#rename_command(new_match, search, options = {}) ⇒ Object

Rename a command. Accepts either match or listing for the search.

Examples:

Renaming the ls command and changing its description.

Pry.config.commands.rename "dir", "ls", :description => "DOS friendly ls"

Parameters:

  • new_match (String, Regexp)

    The new match for the command.

  • search (String, Regexp)

    The command's current match or listing.

  • options (Hash) (defaults to: {})

    The optional configuration parameters, accepts the same as the command method, but also allows the command description to be passed this way too.



250
251
252
253
254
255
256
257
258
259
260
261
262
263
# File 'lib/pry/command_set.rb', line 250

def rename_command(new_match, search, options={})
  cmd = find_command_by_match_or_listing(search)

  options = {
    :listing     => new_match,
    :description => cmd.description
  }.merge!(options)

  commands[new_match] = cmd.dup
  commands[new_match].match = new_match
  commands[new_match].description = options.delete(:description)
  commands[new_match].options.merge!(options)
  commands.delete(cmd.match)
end

#run_command(context, match, *args) ⇒ Object



346
347
348
349
# File 'lib/pry/command_set.rb', line 346

def run_command(context, match, *args)
  command = commands[match] or raise NoCommandError.new(match, self)
  command.new(context).call_safely(*args)
end

#valid_command?(val) ⇒ Boolean

Is the given line a command invocation?

Parameters:

  • val (String)

Returns:

  • (Boolean)


327
328
329
# File 'lib/pry/command_set.rb', line 327

def valid_command?(val)
  !!find_command(val)
end